import sys
sys.path.insert(0,'..')
from chipmunkdb.ChipmunkDb import ChipmunkDb
import time
import pandas as pd


def loadNewDf():
    import duckdb
    import io
    con = duckdb.connect(database='minutelytestdata.duck', read_only=False)
    rel = con.execute("SELECT * FROM parquet_scan('minutelytestdata.parquet');")
    df = rel.df()

    df['Datetime'] = pd.to_datetime(df['__index_level_0__'])
    df = df.set_index(['Datetime'])

    return df

testdata = loadNewDf()

db = ChipmunkDb("localhost", 8091)

collections = db.collections()

print("Currently collections", collections)


collection = db.collection_info("testss")

if collection is not None and "rows" in collection:
    db.dropCollection("testss")

collection = db.collection_info("testss")

if collection is None or "rows" not in collection or collection["rows"] <= 0:
    # we create the empty colleciton
    df = testdata
    start = time.time()
   ## subset = df.loc[:, ["main:open", "main:close", "main:high", "main:low", "main:volume"]]
    db.save_as_pandas(df[["main:open", "main:close", "main:high", "main:low", "main:volume"]], "testss")
    end = time.time()
    print("create ohlc dataframe", end - start)


### lets read the data from the collection

start = time.time()
df = db.collection_as_pandas("testss")
end = time.time()
print("read dataframe", end - start)


start = time.time()
db.save_as_pandas(testdata[["ema_1:y"]], "testss", mode="append")
end = time.time()
print("appended two indicator columns", end - start)

start = time.time()
df = db.collection_as_pandas("testss")
end = time.time()
print("read dataframe with indicator again", end - start)


start = time.time()
half_indi = testdata[["color"]].tail(3)
db.save_as_pandas(half_indi, "testss", mode="append")
end = time.time()
print("appended only head(10) indicator column by datetime", end - start)

start = time.time()
df = db.collection_as_pandas("testss")
end = time.time()
print("read dataframe with half indicator again", end - start)

start = time.time()
half_indi = testdata[["color"]].head(3)
db.save_as_pandas(half_indi, "testss", mode="append")
end = time.time()
print("appended only tail(10) half indicator column by datetime", end - start)

start = time.time()
df = db.collection_as_pandas("testss")
end = time.time()
print("read dataframe with half indicator again", end - start)
print(df)

start = time.time()
d = db.query("SELECT * FROM testss LIMIT 10000")
end = time.time()
print("extract only a part out of it", end - start)




import sys
sys.path.insert(0,'..')
from chipmunkdb.ChipmunkDb import ChipmunkDb
import time
import pandas as pd
from influxdb import DataFrameClient
from influxdb import InfluxDBClient

import duckdb
import io

db = ChipmunkDb("localhost", 8091)

db.dropColumn("testss", "color")


db.dropColumn("testdomain", "*", domain="chart1")


from typing import List

import requests
import io
import pyarrow as pa
import pyarrow.parquet as pq
import json
import pandas as pd


class StorageEntry:
    key: str
    value: str
    datetime: str
    tags: [str]

class FilterEntry:
    key: str
    tags: [str]


class ChipmunkDb():
    def __init__(self, host, port=8091, autoConnect=True):
        self.host = host
        self.port = port
        self.autoConnect = autoConnect
        self.initialize()

    def connect(self):

        return True

    def initialize(self):
        if (self.autoConnect):
            self.connect()
        return True

    def getHostAndPort(self):
        return 'http://'+str(self.host)+":"+str(self.port)

    def create_from_pandas(self, collection, df):
        return self.savePandas(collection, df, mode="create")

    def dropCollection(self, collection):
        res = requests.delete(url=self.getHostAndPort()+"/collection/"+collection+"/drop", data={"collection": collection, "drop": True})
        return res.json()

    def collections(self):
        res = requests.get(url=self.getHostAndPort()+"/collections")
        return res.json()["collections"]

    def collection_info(self, collection):
        res = requests.get(url=self.getHostAndPort()+"/collection/"+collection)
        data = res.json()
        if "error" in data:
            return None
        return data["collection"]

    def get_document(self, directory, document):
        res = requests.get(url=self.getHostAndPort()+"/directory/"+directory+"/"+document)
        retdata = res.json()
        return retdata["data"]

    def save_document(self, directory, document, data, type="single"):
        res = requests.post(url=self.getHostAndPort()+"/directory/"+directory+"/"+document, data=json.dumps({"data": data, "type": type}))
        retdata = res.json()
        if "error" in retdata:
            return None
        return retdata["data"]

    def generate_random_id(self):
        import random
        import string
        letters = string.ascii_lowercase
        return ''.join(random.choice(letters) for i in range(10))

    def save_as_pandas(self, df, collection, mode="append", domain=None, session_id=None, end=False):

        # if df is larger than 500MB, we iterate over it
        if df.memory_usage().sum() > 500000000:
            # lets iterate over the dataframe and send chunks with a session_id
            session_id = self.generate_random_id()

            while True:
                chunk = df.head(60000)
                if chunk.empty:
                    break
                df = df.tail(-60000)
                self.save_as_pandas(chunk, collection, mode=mode, domain=domain, session_id=session_id, end=df.shape[0] == 0)

        else:

            f = io.BytesIO()
            table = pa.Table.from_pandas(df)
            pq.write_table(table, f)

            headerData = {"mode": mode,  "domain": domain}
            if session_id is not None:
                headerData["session_id"] = session_id
                headerData["end"] = end

            f.seek(0, 0)
            res = requests.post(url=self.getHostAndPort()+'/collection/' + str(collection) + '/insertRaw',
                                files={"data": f.getvalue()},
                                headers={"Content-Type": 'application/octet-stream', "x-data": json.dumps(headerData)})
            f.close()

        return True

    def query(self, query, domain=None):
        domainQ = ""
        if domain is not None:
            domainQ = "&domain="+domain
        res = requests.get(url=self.getHostAndPort()+"/query?q="+query+domainQ, headers={"x-data": json.dumps({"streamed-response": "true"})})
        data = res.json()

        return data["result"]

    def dropColumn(self, collection, columns, domain=None):
        res = requests.delete(url=self.getHostAndPort() + "/collection/" + collection + "/dropColumns",
                           headers={"Content-Type": 'application/octet-stream',
                                    "x-data": json.dumps({"columns": columns, "domain": domain})})
        data = res.content

        return data

    def collection_as_pandas(self, collection, columns=[], domain=None):
        res = requests.get(url=self.getHostAndPort()+"/collection/"+collection+"/rawStream",
                           headers={"Content-Type": 'application/octet-stream', "x-data": json.dumps({"columns": columns, "domain": domain})})
        data = res.content

        pq_file = io.BytesIO(data)
        df = pd.read_parquet(pq_file)

        return df

    def collection_as_pandas_additional(self, collection, additionalCollections: [], columns=[], domain=None):
        res = requests.get(url=self.getHostAndPort()+"/collection/"+collection+"/rawStream",
                           headers={"Content-Type": 'application/octet-stream', "x-data": json.dumps({"columns": columns,
                                                                                                      "additionalCollections": additionalCollections,
                                                                                                      "domain": domain})})
        data = res.content

        pq_file = io.BytesIO(data)
        df = pd.read_parquet(pq_file)

        return df

    def storages(self):
        res = requests.get(url=self.getHostAndPort()+"/storages")
        return res.json()["storages"]

    def drop_storage(self, storage):
        res = requests.delete(url=self.getHostAndPort()+"/storage/"+storage)
        return res.json()["success"]

    def get_storage(self, storage):
        res = requests.get(url=self.getHostAndPort()+"/storage/"+storage)
        return res.json()["data"]

    def filter_storage(self, storage, filters: List[FilterEntry]):
        res = requests.post(url=self.getHostAndPort()+"/storage/"+storage+"/filter", data=json.dumps(filters))
        if res.status_code != 200:
            raise(res.json()["message"])
        return res.json()["data"]


    def save_keys_to_storage(self, storage, keys: List[StorageEntry]):
        res = requests.post(url=self.getHostAndPort()+"/storage/"+storage, data=json.dumps(keys))
        return res.json()["count"]

    def get_key_for_storage(self, storage, key):
        res = requests.get(url=self.getHostAndPort()+"/storage/"+storage+"/"+key)
        return res.json()["data"]

def main():
    pass


import sys
sys.path.insert(0,'..')
from chipmunkdb.ChipmunkDb import ChipmunkDb
import time


db = ChipmunkDb("localhost", 8091)

start = time.time()
document = db.collection_as_pandas_additional("dev_602be966a7a840a2e232d6eb", ["dev_602be966a7a840a2e232d6eb_6060b1c1375dcc2a5f90bb08"])

print("doc", document)


print("time", time.time() - start)



import sys
sys.path.insert(0,'..')
from chipmunkdb.ChipmunkDb import ChipmunkDb
import time


db = ChipmunkDb("localhost", 8091)

start = time.time()
db.save_document("asd", "window", [{"name": "Thoren", "age": 32}])

document = db.get_document("asd", "window")



print("time", time.time() - start)



import sys
sys.path.insert(0,'..')
from chipmunkdb.ChipmunkDb import ChipmunkDb
import time
import pandas as pd

import duckdb
import io

db = ChipmunkDb("localhost", 8091)


storages = db.storages()
print("all storages", storages)

entries = [
    {"key": "cmc.rank", "value": 12},
    {"key": "cmc.rank", "value": 12, "tags": ["ETH"]}
]

db.save_keys_to_storage("test_code", entries)

all_data = db.get_storage("test_code")
print("al", all_data)

eth_key = db.filter_storage("test_code", {"key": "cmc.rank", "tags": ["ETH"]})
print("keye", eth_key)

non_eth_key = db.filter_storage("test_code", {"key": "cmc.rank"})
print("keye", non_eth_key)






import sys
sys.path.insert(0,'..')
from chipmunkdb.ChipmunkDb import ChipmunkDb
import time



db = ChipmunkDb("localhost", 8091)

start = time.time()
df = db.collection_as_pandas("5f9414e059463e65cadf356c")
print("time", time.time() - start)
print(df)


"""Setup script for chipmunkdb_python_client"""

import os.path
from setuptools import setup

# The directory containing this file
HERE = os.path.abspath(os.path.dirname(__file__))

# The text of the README file
with open(os.path.join(HERE, "README_pip.md")) as fid:
    README = fid.read()

# This call to setup() does all the work
setup(
    name="chipmunkdb_python_client",
    version="2.0.4",
    description="Read and Write Dataframes and Data to a chipmunkdb",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/coindeck/chipmunkdb-python-client",
    author="coindeck",
    author_email="donnercody86@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
    ],
    packages=["chipmunkdb"],
    include_package_data=True,
    install_requires=[
        "requests", "pandas", "influxdb", "importlib_resources", "typing"
    ],
    entry_points={"console_scripts": ["chipmunkdb_python_client=chipmunkdb.ChipmunkDb:main"]},
)


import sys
sys.path.insert(0,'..')
from chipmunkdb.ChipmunkDb import ChipmunkDb
import time
import pandas as pd
from influxdb import DataFrameClient
from influxdb import InfluxDBClient

import duckdb
import io

### loading all samples for domains
con = duckdb.connect(database='testdata.duck', read_only=False)
rel = con.execute("SELECT * FROM parquet_scan('minutelytestdata.parquet');")
testdata = rel.df()
testdata['Datetime'] = pd.to_datetime(testdata['__index_level_0__'])
testdata = testdata.set_index(['Datetime'])

rel = con.execute("SELECT * FROM parquet_scan('fiveminutetestdata.parquet');")
fiveminutetestdata = rel.df()
fiveminutetestdata['Datetime'] = pd.to_datetime(fiveminutetestdata['__index_level_0__'])
fiveminutetestdata = fiveminutetestdata.set_index(['Datetime'])

rel = con.execute("SELECT * FROM parquet_scan('hourlytestdata.parquet');")
hourlytestdata = rel.df()
hourlytestdata['Datetime'] = pd.to_datetime(hourlytestdata['__index_level_0__'])
hourlytestdata = hourlytestdata.set_index(['Datetime'])

db = ChipmunkDb("localhost", 8091)

collections = db.collections()

print("Currently collections", collections)

collection = db.collection_info("testdomain")

if collection is not None and "rows" in collection:
    db.dropCollection("testdomain")

collection = db.collection_info("testdomain")

start = time.time()
## subset = df.loc[:, ["main:open", "main:close", "main:high", "main:low", "main:volume"]]
db.save_as_pandas(testdata[["main:open", "main:close", "main:high", "main:low", "main:volume"]], "testdomain", domain="chart1")
db.save_as_pandas(fiveminutetestdata[["main:open", "main:close", "main:high", "main:low", "main:volume"]], "testdomain", domain="chart2")
db.save_as_pandas(hourlytestdata[["main:open", "main:close", "main:high", "main:low", "main:volume"]], "testdomain", domain="chart3")
end = time.time()
print("create ohlc dataframe", end - start)


### lets read the data from the collection

start = time.time()
df = db.collection_as_pandas("testdomain", domain="chart1")
end = time.time()
print("read dataframe", end - start)

print(df)

start = time.time()
db.save_as_pandas(testdata[["ema_1:y"]], "testdomain", mode="append", domain="chart1")
end = time.time()
print("appended two indicator columns", end - start)

start = time.time()
df = db.collection_as_pandas("testdomain", domain="chart1")
end = time.time()
print("read dataframe with indicator again", end - start)

print(df)


start = time.time()
df = db.collection_as_pandas("testdomain")
end = time.time()
print("read complete dataframe with indicator again", end - start)

print(df)




start = time.time()
half_indi = testdata[["color"]].tail(3)
db.save_as_pandas(half_indi, "testdomain", mode="append", domain="chart1")
end = time.time()
print("appended only head(10) indicator column by datetime", end - start)

start = time.time()
df = db.collection_as_pandas("testdomain")
end = time.time()
print("read dataframe with half indicator again", end - start)

start = time.time()
half_indi = testdata[["color"]].head(3)
db.save_as_pandas(half_indi, "testdomain", mode="append", domain="chart1")
end = time.time()
print("appended only tail(10) half indicator column by datetime", end - start)

start = time.time()
df = db.collection_as_pandas("testdomain", domain="chart1")
end = time.time()
print("read dataframe with half indicator again", end - start)
print(df)

start = time.time()
d = db.query("SELECT * FROM testdomain LIMIT 10000", domain="chart1")
end = time.time()
print("extract only a part out of it", end - start)




import numpy as np
import pandas as pd

import duckdb
import io

category_idx = pd.Index(['A', 'B'])
date_idx = pd.date_range('2018-01', '2018-02', freq='MS')
idx = pd.MultiIndex.from_product([category_idx, date_idx], names=['category', 'date'])

series = pd.Series(np.random.randn(len(category_idx) * len(date_idx)), index=idx)


df = pd.DataFrame(series)

print(df)
con = duckdb.connect(database='multiindex.duck', read_only=False)

con.register("my_view", df)

con.execute("SELECT * FROM my_view")

print(con.fetchall())


import sys
sys.path.insert(0,'..')
from chipmunkdb.ChipmunkDb import ChipmunkDb
import time
import pandas as pd

import duckdb
import io

db = ChipmunkDb("localhost", 8091)

collections = db.collections()

df = db.collection_as_pandas("testdomain")


# resize the dataframe by copying it to more than 500mb
# this will cause the dataframe to be split into multiple chunks
# and the dataframe will be split into multiple chunksw
last_date = df.index[-1]
while df.memory_usage().sum() < 500000000:
    data_index = []
    data = []
    for i in range(0, 100000):
        last_date = last_date + pd.Timedelta(minutes=1)
        data_index.append(last_date)
        data.append({"datetime": last_date, "chart2.main:open": 1, "chart2.main:close": 1,
                                                              "chart2.main:high": 1, "chart2.main:low": 1, "chart2.main:volume": 1})

    df = pd.concat([df, pd.DataFrame(index=data_index, data=data)])



start = time.time()
db.save_as_pandas(df, "testdomain", mode="dropbefore")
end = time.time()
print("read complete dataframe with indicator again", end - start)






start = time.time()
d = db.query("SELECT * FROM testdomain LIMIT 10000", domain="chart1")
end = time.time()
print("extract only a part out of it", end - start)

